package org.objectweb.telosys.dal.dao;

import java.sql.Connection;
import java.sql.SQLException;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;

/**
 * @author Laurent GUERIN
 *  
 */
public class StandardDatabaseSession extends TelosysObject implements DatabaseSession
{
    int        _iDatabaseId         = 0;

    Connection _con                 = null;

    boolean    _bOriginalAutoCommit = false;

    /**
     * Constructor for a real database dession
     * 
     * @param iDatabaseId
     * @param con
     * @throws TelosysException
     */
    public StandardDatabaseSession(int iDatabaseId, Connection con) throws TelosysException
    {
        super();
        if (iDatabaseId < 0)
        {
            throw new TelosysException("DatabaseSession constructor : database id < 0 ( " + iDatabaseId + ") ");
        }
        if (con == null)
        {
            throw new TelosysException("DatabaseSession constructor : connection is null ");
        }
        // In a session, the connection must not be in "AutoCommit mode"
        try
        {
            //--- Keep the original AutoCommit state
            _bOriginalAutoCommit = con.getAutoCommit();
            //--- Set the Programmatic Transaction control
            con.setAutoCommit(false);
        } catch (SQLException ex)
        {
            throw new TelosysException("DatabaseSession constructor : cannot get/set AutoCommit", ex);
        }
        _iDatabaseId = iDatabaseId;
        _con = con;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.dal.dao.DatabaseSession#isDummy()
     */
    public boolean isDummy()
    {
        return false;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.dal.dao.DatabaseSession#getDatabaseId()
     */
    public int getDatabaseId()
    {
        return _iDatabaseId;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.dal.dao.DatabaseSession#getConnection()
     */
    public Connection getConnection()
    {
        return _con;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.dal.dao.DatabaseSession#commit()
     */
    public void commit() throws TelosysException
    {
            if (_con == null)
            {
                throw new TelosysException("DatabaseSession.commit() : connection is null");
            }
            else
            {
                try
                {
                    _con.commit();
                } catch (SQLException e)
                {
                    throw new TelosysException("DatabaseSession.commit() : Cannot commit", e);
                }
            }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.dal.dao.DatabaseSession#rollback()
     */
    public void rollback() throws TelosysException
    {
            if (_con == null)
            {
                throw new TelosysException("DatabaseSession.rollback() : connection is null");
            }
            else
            {
                try
                {
                    _con.rollback();
                } catch (SQLException e)
                {
                    throw new TelosysException("DatabaseSession.rollback() : Cannot rollback", e);
                }
            }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.dal.dao.DatabaseSession#close()
     */
    public void close() throws TelosysException
    {
            if (_con == null)
            {
                throw new TelosysException("DatabaseSession.close() : connection is null");
            }
            else
            {
                try
                {
                    //--- Reset the original AutoCommit state
                    _con.setAutoCommit(_bOriginalAutoCommit);
                    //--- Release the connection
                    _con.close();
                } catch (SQLException e)
                {
                    throw new TelosysException("DatabaseSession.close() : connection error on setAutoCommit or close ", e);
                }
            }
    }
}