package org.objectweb.telosys.rpl;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.LinkedList;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.telosys.common.TelosysConst;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.screen.core.ScreenData;
import org.objectweb.telosys.screen.core.ScreenDataElements;
import org.objectweb.telosys.screen.core.ScreenError;
import org.objectweb.telosys.screen.core.ScreenMessage;
import org.objectweb.telosys.util.web.GenericURI;

/**
 * @author Laurent GUERIN
 * 
 */
public class ScreenResponse extends TelosysObject
{
    public final static int ACTION_OK       = 0;

    public final static int ACTION_CANCELED = 1;

    public final static int ACTION_ERROR    = 2;

    public final static int INVALID_ACTION    = 3 ;
    
    public final static int INVALID_SCREEN_ID    = 101 ;

    public final static int SCREEN_NOT_FOUND     = 102 ;

    //-----------------------------------------------------------------------------------------------
    // Current http response object 
    //-----------------------------------------------------------------------------------------------

    private HttpServletRequest  _httpRequest  = null ; // since v 0.9.9
    
    private HttpServletResponse _httpResponse = null ; // since v 0.9.9
    
    //-----------------------------------------------------------------------------------------------
    // Screen Request parameters
    //-----------------------------------------------------------------------------------------------

    private String          _sScreenName    = null;

    private String          _sScreenId      = null;

    private String          _sAction        = null;

    private String          _sProcName      = null;

    //-----------------------------------------------------------------------------------------------
    // Screen Response data
    //-----------------------------------------------------------------------------------------------
    ScreenDataElements      _dataElements   = null ; // Data elements to return in the response
    
    //private ScreenData      _Data           = null; // Data to return in the response

    private Hashtable       _htValues       = null; // Values to return in the response

    private LinkedList      _listExceptions = null; // List of all the exceptions

    private LinkedList      _listErrors     = null; // List of all the errors

    private LinkedList      _listMessages   = null; // List of all the app. messages

    private int             _iReturnCode    = 0;    // default = "0" ( = OK )

    private String          _sReturnMessage = "Ok"; // default = "Ok"

    //private boolean _bNotFound = false;

    private boolean         _bFlagNotFound  = false;

    private boolean         _bFlagFound     = false;
    
    private String          _sRedirection   = null ; // v 0.9.9

    //-----------------------------------------------------------------------------------------------
    // constructor(s)
    //-----------------------------------------------------------------------------------------------

// Removed in v 0.9.9
//    public ScreenResponse(String sScreenName, String sScreenId, String sAction, String sProcName)
//    {
//        _sScreenName = sScreenName;
//        _sScreenId = sScreenId;
//        _sAction = sAction;
//        _sProcName = sProcName;
//    }

    // New constructor since v 0.9.9
    /**
     * Constructor
     * @param httpRequest
     * @param httpResponse
     * @param sScreenName
     * @param sScreenId
     * @param sAction
     * @param sProcName
     */
    public ScreenResponse(HttpServletRequest httpRequest, HttpServletResponse httpResponse, String sScreenName, String sScreenId, String sAction, String sProcName)
    {
        _httpRequest = httpRequest ;
        _httpResponse = httpResponse;
        _sScreenName = sScreenName;
        _sScreenId = sScreenId;
        _sAction = sAction;
        _sProcName = sProcName;
    }

    //-----------------------------------------------------------------------------------------------
    // methods
    //-----------------------------------------------------------------------------------------------

    //-----------------------------------------------------------------------------------------------
    public String getScreenName()
    {
        return _sScreenName;
    }

    //-----------------------------------------------------------------------------------------------
    public String getScreenId()
    {
        return _sScreenId;
    }

    //-----------------------------------------------------------------------------------------------
    public String getAction()
    {
        return _sAction;
    }

    //-----------------------------------------------------------------------------------------------
    public String getProcName()
    {
        return _sProcName;
    }

    //-----------------------------------------------------------------------------------------------
    public void setReturnCode(int iCode)
    {
        _iReturnCode = iCode;
    }

    //-----------------------------------------------------------------------------------------------
    public int getReturnCode()
    {
        return _iReturnCode;
    }

    //-----------------------------------------------------------------------------------------------

    //-----------------------------------------------------------------------------------------------
    public void setActionReturn(int iActionReturnCode)
    {
        switch (iActionReturnCode)
        {
        case ACTION_OK :
            _iReturnCode = ACTION_OK;
            _sReturnMessage = "OK";
            break;
        case ACTION_CANCELED :
            _iReturnCode = ACTION_CANCELED;
            _sReturnMessage = "Action canceled";
            break;
        case ACTION_ERROR :
            _iReturnCode = ACTION_ERROR;
            _sReturnMessage = "Action error (abnormal termination)";
            break;
        default :
            _iReturnCode = 99999;
            _sReturnMessage = "Unknown action return code " + iActionReturnCode;
            break;
        }
    }

    //-----------------------------------------------------------------------------------------------
    public void setReturnMessage(String sMsg)
    {
        _sReturnMessage = sMsg;
    }

    //-----------------------------------------------------------------------------------------------
    public String getReturnMessage()
    {
        return _sReturnMessage;
    }

    //-----------------------------------------------------------------------------------------------

    //-----------------------------------------------------------------------------------------------
    //    public void setFlagNotFound(boolean bFlag)
    //    {
    //        _bNotFound = bFlag;
    //    }

    //-----------------------------------------------------------------------------------------------
    // Flag : return a tag "not found" or no
    //-----------------------------------------------------------------------------------------------
    public boolean isFlagNotFound()
    {
        return _bFlagNotFound;
    }

    //-----------------------------------------------------------------------------------------------
    public void setFlagNotFound()
    {
        _bFlagNotFound = true;
        _bFlagFound = false;
    }

    //-----------------------------------------------------------------------------------------------
    public void unsetFlagNotFound()
    {
        _bFlagNotFound = false;
    }

    //-----------------------------------------------------------------------------------------------
    // Flag : return a tag "found" or no
    //-----------------------------------------------------------------------------------------------
    public boolean isFlagFound()
    {
        return _bFlagFound;
    }

    //-----------------------------------------------------------------------------------------------
    public void setFlagFound()
    {
        _bFlagFound = true;
        _bFlagNotFound = false;
    }

    //-----------------------------------------------------------------------------------------------
    public void unsetFlagFound()
    {
        _bFlagFound = false;
    }

    //-----------------------------------------------------------------------------------------------
    // Messages
    //-----------------------------------------------------------------------------------------------
    public void addMessage(ScreenMessage message)
    {
        if (_listMessages == null)
        {
            _listMessages = new LinkedList();
        }

        if (message != null)
        {
            _listMessages.add(message);
        }
    }

    //-----------------------------------------------------------------------------------------------
    public int getNumberOfMessages()
    {
        if (_listMessages == null)
        {
            return 0;
        }
        else
        {
            return _listMessages.size();
        }
    }

    //-----------------------------------------------------------------------------------------------
    public ScreenMessage getMessage(int index)
    {
        if (_listMessages == null)
        {
            return null;
        }
        else
        {
            Object obj = _listMessages.get(index);
            if (obj instanceof ScreenMessage)
            {
                return (ScreenMessage) obj;
            }
            else
            {
                error("getMessage(" + index + ") : not a ScreenMessage instance ");
                return null;
            }
        }
    }

    //-----------------------------------------------------------------------------------------------
    // Errors
    //-----------------------------------------------------------------------------------------------
    public void addError(ScreenError error)
    {
        if (_listErrors == null)
        {
            _listErrors = new LinkedList();
        }

        if (error != null)
        {
            _listErrors.add(error);
        }
    }

    //-----------------------------------------------------------------------------------------------
    public int getNumberOfErrors()
    {
        if (_listErrors == null)
        {
            return 0;
        }
        else
        {
            return _listErrors.size();
        }
    }

    //-----------------------------------------------------------------------------------------------
    public ScreenError getError(int index)
    {
        if (_listErrors == null)
        {
            return null;
        }
        else
        {
            Object obj = _listErrors.get(index);
            if (obj instanceof ScreenError)
            {
                return (ScreenError) obj;
            }
            else
            {
                error("getError(" + index + ") : not a ScreenError instance ");
                return null;
            }
        }
    }

    //-----------------------------------------------------------------------------------------------
    // Exceptions
    //-----------------------------------------------------------------------------------------------
//    public void addError(Exception ex)
//    {
//        if (_listExceptions == null)
//        {
//            _listExceptions = new LinkedList();
//        }
//
//        if (ex != null)
//        {
//            _listExceptions.add(ex);
//        }
//    }

    public void addError(Throwable th)
    {
        if (_listExceptions == null)
        {
            _listExceptions = new LinkedList();
        }

        if (th != null)
        {
            _listExceptions.add(th);
        }
    }

    //-----------------------------------------------------------------------------------------------
    public int getNumberOfExceptions()
    {
        if (_listExceptions == null)
        {
            return 0;
        }
        else
        {
            return _listExceptions.size();
        }
    }

    //-----------------------------------------------------------------------------------------------
    //public Exception getException(int index)
    public Throwable getException(int index)
    {
        if (_listExceptions == null)
        {
            return null;
        }
        else
        {
            Object obj = _listExceptions.get(index);
            if (obj instanceof Throwable)
            {
                return (Throwable) obj;
            }
            else
            {
                error("getException(" + index + ") : not a Throwable instance ");
                return null;
            }
        }
    }

    //-----------------------------------------------------------------------------------------------
    // Data
    //-----------------------------------------------------------------------------------------------
//    public ScreenData getData()
//    {
//        return _Data;
//    }
//
//    //-----------------------------------------------------------------------------------------------
//    public void setData(ScreenData data)
//    {
//        _Data = data;
//    }
    //-----------------------------------------------------------------------------------------------
    public ScreenDataElements getDataElements()
    {
//        if ( _Data != null )
//        {
//            return _Data.getElements() ;            
//        }
//        return null ;
        return _dataElements ;
    }
    //-----------------------------------------------------------------------------------------------
    public void setDataElements(ScreenData data)
    {
      if ( data != null )
      {
          _dataElements = data.getElements() ;            
      }
      else
      {
          _dataElements = null ;                      
      }
    }
    //-----------------------------------------------------------------------------------------------
    public void setDataElements(ScreenDataElements dataElements)
    {
        _dataElements = dataElements;
    }

    //-----------------------------------------------------------------------------------------------
    // Values
    //-----------------------------------------------------------------------------------------------
    public void setValue(String sName, String sValue)
    {
        if (_htValues == null)
        {
            _htValues = new Hashtable();
        }

        if (sName != null)
        {
            _htValues.put(sName, sValue);
        }
    }

    //-----------------------------------------------------------------------------------------------
    public void setValue(String sName, int iValue)
    {
        if (_htValues == null)
        {
            _htValues = new Hashtable();
        }

        if (sName != null)
        {
            _htValues.put(sName, ""+iValue);
        }
    }

    //-----------------------------------------------------------------------------------------------
    public String getValue(String sName)
    {
        if (_htValues == null)
        {
            return null;
        }
        else
        {
            return (String) _htValues.get(sName);
        }
    }

    //-----------------------------------------------------------------------------------------------
    public int getNumberOfValues()
    {
        if (_htValues == null)
        {
            return 0;
        }
        else
        {
            return _htValues.size();
        }
    }

    //-----------------------------------------------------------------------------------------------
    public Enumeration getValueNames()
    {
        if (_htValues == null)
        {
            return null;
        }
        else
        {
            return _htValues.keys();
        }
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Force the client-side (the browser) to redirect to the given URL <br>
     * ( replace the current screen by the given URL )
     * @param sURL
     */
    public void forceRedirect ( String sURL )
    {
        if ( sURL != null )
        {
            if ( _httpResponse != null )
            {
                _sRedirection = sURL ;
                _httpResponse.setHeader(TelosysConst.TELOSYS_REDIRECT, _sRedirection );
            }
            else
            {
                throw new TelosysRuntimeException("forceRedirect : HttpResponse is null");
            }
        }
    }
    
    //-----------------------------------------------------------------------------------------------
    /**
     * Force the client-side (the browser) to redirect to the given generic URI <br>
     * ( replace the current screen by the given object URL )
     * @param uri
     */
    public void forceRedirect ( GenericURI uri )
    {
        if ( uri != null )
        {
            forceRedirect( uri.getFullURL( _httpRequest ) );
        }
    }
    
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the redirection URL set by 'forceRedirect' or null if none
     * @return
     */
    public String getRedirection()
    {
        return _sRedirection ;
    }

    //-----------------------------------------------------------------------------------------------
    //    public void storeDataElement(Object objData)
    //    {
    //        if (_Data != null)
    //        {
    //            _Data.store(objData);
    //        }
    //    }

    //-----------------------------------------------------------------------------------------------
    public String toString()
    {
        //--- Attributes
        String sHeader = _sScreenName + "|" + _sScreenId + "|" + _sAction + "|" + _sProcName;

        //--- Data
        String sData = "(no data)";
//        if (_Data != null)
//        {
//            sData = _Data.toString();
//        }

        return "ScreenResponse : " + sHeader + "\n Data = " + sData;
    }
    //-----------------------------------------------------------------------------------------------
}